@@ -39,7 +39,7 @@ import networkx as nx
 from neo4j import GraphDatabase
 
 from .abc_property_graph import ABCPropertyGraph, PropertyGraphImportException, \
-    PropertyGraphQueryException, ABCGraphImporter
+    PropertyGraphQueryException, ABCGraphImporter, GraphFormat
 
 # to deal with intermittent APOC problems on MAC
 APOC_RETRY_COUNT = 10
@@ -107,12 +107,23 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
                                                   node_id=None, msg="Unable to find network links")
             return val.data()['nodeids']
 
+    def get_all_nodes_by_class_and_type(self, *, label: str, ntype: str) -> List[str]:
+        assert label is not None
+        assert ntype is not None
+        query = f"MATCH(n:{label} {{GraphID: $graphId, Type: $ntype}}) RETURN collect(n.NodeID) as nodeids"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id, ntype=ntype).single()
+            if val is None:
+                raise PropertyGraphQueryException(graph_id=self.graph_id,
+                                                  node_id=None, msg="Unable to find network links")
+            return val.data()['nodeids']
+
     def list_all_node_ids(self) -> List[str]:
         """
         List all NodeID properties of nodes in a graph
         :return:
         """
-        query = "MATCH (n {GraphID: $graphId}) RETURN collect(n.NodeID) as nodeids"
+        query = "MATCH (n:GraphNode {GraphID: $graphId}) RETURN collect(n.NodeID) as nodeids"
         with self.driver.session() as session:
             val = session.run(query, graphId=self.graph_id).single()
             if val is None:
@@ -322,12 +333,15 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
                                                   node_b=node_b, kind=kind,
                                                   msg="Unable to set properties on a link")
 
-    def serialize_graph(self) -> str:
+    def serialize_graph(self, format: GraphFormat = GraphFormat.GRAPHML) -> str:
         """
         Serialize a given graph into GraphML string or return None if graph not found
         :return:
         """
-        inner_query = f'match(n {{GraphID: "{self.graph_id}"}}) optional match(n) -[r]- (m) return n, r, m'
+        if format != GraphFormat.GRAPHML:
+            PropertyGraphQueryException(graph_id=self.graph_id, node_id=None,
+                                        msg=f"Unsupported export graph format {format.name}")
+        inner_query = f'match(n:GraphNode {{GraphID: "{self.graph_id}"}}) optional match(n) -[r]- (m) return n, r, m'
         # run inner query to check the graph has anything in it
         with self.driver.session() as session:
             val = session.run(inner_query)
@@ -362,7 +376,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         Does the graph with this ID exist?
         :return:
         """
-        inner_query = f'match(n {{GraphID: "{self.graph_id}"}}) -[r]- (m) return n, r, m'
+        inner_query = f'match(n:GraphNode {{GraphID: "{self.graph_id}"}}) -[r]- (m) return n, r, m'
         # run  query to check the graph has anything in it
         with self.driver.session() as session:
             val = session.run(inner_query)
@@ -438,7 +452,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
 
         query = f"match (a:GraphNode {{GraphID: $graphId, NodeID: $nodeA}}) -[:{rel1}]- "\
                 f"(b:{node1_label} {{GraphID: $graphId}}) -[:{rel2}]- "\
-                f"(c:{node2_label} {{GraphID: $graphId}}) return b.NodeID, c.NodeID"
+                f"(c:{node2_label} {{GraphID: $graphId}}) WHERE $nodeA <> c.NodeID return b.NodeID, c.NodeID"
         with self.driver.session() as session:
             val = session.run(query, graphId=self.graph_id, nodeA=node_id).values()
             if val is None:
@@ -568,9 +582,31 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
                         graphId1=other_graph.graph_id,
                         nodeId=node_id).single()
 
+    def get_stitch_nodes(self) -> List[str]:
+        query = f"MATCH (n:GraphNode {{GraphID: $graphId, StitchNode: 'true'}}) RETURN collect(n.NodeID) as nodeids"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id).single()
+            if val is None:
+                raise PropertyGraphQueryException(graph_id=self.graph_id,
+                                                  node_id=None, msg="Unable to find stitch nodes")
+            return val.data()['nodeids']
+
+    def check_node_unique(self, *, label: str, name: str) -> bool:
+        assert label is not None
+        assert name is not None
+
+        query = f"MATCH (n:{label} {{GraphID: $graphId, Name: $name}}) RETURN collect(n.NodeID) as nodeids"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id, name=name).single()
+            if val is None or len(val.data()) == 0 or len(val.data()['nodeids']) == 0:
+                return True
+            return False
+
 
 class Neo4jGraphImporter(ABCGraphImporter):
 
+    index_initialized = False
+
     def __init__(self, *, url: str, user: str, pswd: str, import_host_dir: str, import_dir: str, logger=None):
         """
         URL of Neo4j instance, credentials and directory
@@ -596,6 +632,28 @@ class Neo4jGraphImporter(ABCGraphImporter):
             self.log = logging.getLogger(__name__)
         else:
             self.log = logger
+        self._add_indexes()
+
+    def _add_indexes(self):
+        """
+        Add required indexes in idempotent manner
+        :return:
+        """
+        if Neo4jGraphImporter.index_initialized:
+            return
+        Neo4jGraphImporter.index_initialized = True
+        index_file = os.path.join(os.path.dirname(__file__), 'data', 'neo4j_indexes.json')
+        f = open(index_file)
+        index_dict = json.load(f)
+        f.close()
+        self.log.info('Adding Neo4j indexes')
+        for index_name, index_cmd in index_dict.items():
+            with self.driver.session() as session:
+                try:
+                    session.run(index_cmd)
+                except:
+                    # ignore exceptions
+                    pass
 
     def _prep_graph(self, graph: str, graph_id: str = None) -> Tuple[str, str, str]:
         """
