@@ -26,19 +26,23 @@
 import enum
 import ipaddress
 
+from fim.slivers.capacities_labels import Location
 from .base_sliver import BaseSliver
-from .attached_components import AttachedComponentsInfo
-from .interface_info import InterfaceInfo
-from .switch_fabric import SwitchFabricInfo
 
 
 class NodeType(enum.Enum):
+    """
+    Possible NetworkNode types in FABRIC.
+    """
     Server = enum.auto()
     VM = enum.auto()
     Container = enum.auto()
     Switch = enum.auto()
     NAS = enum.auto()
 
+    def help(self) -> str:
+        return 'A ' + self.name
+
     def __repr__(self):
         return self.name
 
@@ -56,9 +60,9 @@ class NodeSliver(BaseSliver):
         self.image_type = None
         self.image_ref = None
         self.service_endpoint = None
-        self.switch_fabric_info = None
+        self.network_service_info = None
         self.site = None
-        self.management_interface_mac_address = None
+        self.location = None
 
     #
     # Setters are only needed for things we want users to be able to set
@@ -102,6 +106,13 @@ class NodeSliver(BaseSliver):
     def get_site(self) -> str:
         return self.site
 
+    def set_location(self, location: Location):
+        assert(location is None or isinstance(location, Location))
+        self.location = location
+
+    def get_location(self) -> Location:
+        return self.location
+
     @staticmethod
     def type_from_str(ntype: str) -> NodeType or None:
         if ntype is None:
