@@ -396,10 +396,11 @@ class Topology(ABC):
         :param graph_string:
         :return:
         """
+        if file_name is None and graph_string is None:
+            raise RuntimeError('Either file_name or graph_string parameters must be specified.')
         if file_name is not None:
             nx_pgraph = self.graph_model.importer.import_graph_from_file_direct(graph_file=file_name)
         else:
-            assert graph_string is not None
             nx_pgraph = self.graph_model.importer.import_graph_from_string_direct(graph_string=graph_string)
         self.graph_model = NetworkxASM(graph_id=nx_pgraph.graph_id,
                                        importer=nx_pgraph.importer,
@@ -568,7 +569,7 @@ class SubstrateTopology(Topology):
     def __copy_to_delegations(e: ModelElement, atype: DelegationType,
                               delegation_id: str) -> Delegations or None:
         """
-        From a model element (node, component, switch_fabric, interface), copy
+        From a model element (node, component, network service, interface), copy
         capacities or labels to a Delegations and return Delegations or none if
         no capacities or labels specified. Return None if element is a stitch node.
         :param e:
@@ -677,10 +678,11 @@ class AdvertizedTopology(Topology):
         :param graph_string:
         :return:
         """
+        if file_name is None and graph_string is None:
+            raise RuntimeError('Either file_name or graph_string must be specified.')
         if file_name is not None:
             nx_pgraph = self.graph_model.importer.import_graph_from_file_direct(graph_file=file_name)
         else:
-            assert graph_string is not None
             nx_pgraph = self.graph_model.importer.import_graph_from_string_direct(graph_string=graph_string)
         self.graph_model = NetworkXABQMFactory.create(nx_pgraph)
 
@@ -779,7 +781,8 @@ class AdvertizedTopology(Topology):
             edge_labels = dict()
             for k, v in graph_edges.items():
                 derived_graph.add_edge(v[0], v[1])
-                edge_labels[(v[0], v[1])] = k
+                #edge_labels[(v[0], v[1])] = k
+                edge_labels[(v[0], v[1])] = ""
 
             pos = layout(derived_graph)
             nx.draw_networkx(derived_graph, pos=pos)
@@ -819,9 +822,13 @@ class AdvertizedTopology(Topology):
         """
         lines = list()
         for n in self.sites.values():
-            ncp = CapacityTuple(total=n.get_property("capacities"),
-                                allocated=n.get_property("capacity_allocations"))
-            lines.append(n.name + ": " + str(ncp))
+            tot_cap = n.get_property("capacities")
+            alloc_cap = n.get_property("capacity_allocations")
+            if tot_cap is not None and alloc_cap is not None:
+                ncp = CapacityTuple(total=tot_cap, allocated=alloc_cap)
+                lines.append(n.name + ": " + str(ncp))
+            else:
+                lines.append(n.name)
             lines.append("\tComponents:")
             for c in n.components.values():
                 ccp = CapacityTuple(total=c.get_property("capacities"),
